@@ -17,10 +17,13 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: popup-blocker.c,v 1.1 2004/01/02 07:12:02 chpe Exp $
+ *  $Id: popup-blocker.c,v 1.2 2004/03/06 00:15:18 adamh Exp $
  */
 
 #include "ephy-popup-blocker-extension.h"
+#include "ephy-popup-blocker-icon.h"
+#include "ephy-popup-blocker-popup.h"
+
 #include "ephy-debug.h"
 
 #include <gmodule.h>
@@ -37,6 +40,9 @@ register_module (GTypeModule *module)
 	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
 #endif /* ENABLE_NLS */
 
+	ephy_popup_blocker_icon_register_type (module);
+	ephy_popup_blocker_popup_register_type (module);
+
 	return ephy_popup_blocker_extension_register_type (module);
 }
 
